using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows.Media;
using Ascon.Pilot.Theme.ColorScheme;
using Ascon.Pilot.SDK.Menu;
using Ascon.Pilot.SDK.TaskSample.Extensions;
using Ascon.Pilot.SDK.TaskSample.Services;

namespace Ascon.Pilot.SDK.TaskSample
{
    [Export(typeof(IMenu<TasksViewContext2>))]
    public class TaskMenuSample : IMenu<TasksViewContext2>
    {
        private const string CREATE_NEW_TASK = "miCreateNewTask";
        private const string CREATE_NEW_WORKFLOW = "miCreateNewWorkflow";
        private const string SHOW_WORKFLOW_DIALOG_WITH_PREDEFINED_DATA = "miShowWorkflowDialogWithPredefinedData";
        private const string SHOW_TASK_DIALOG_WITH_PREDEFINED_DATA = "miShowTaskDialogWithPredefinedData";

        private readonly ITaskTypesService _taskTypesService;
        private readonly IDialogService _dialogService;
        private readonly Dictionary<string, IType> _types = new Dictionary<string, IType>();

        [ImportingConstructor]
        public TaskMenuSample(IPilotServices pilotServices, IDialogService dialogService, ITaskTypesService taskTypesService)
        {
            _dialogService = dialogService;
            _taskTypesService = taskTypesService;

            var accentColor = (Color)ColorConverter.ConvertFromString(pilotServices.PilotDialogService.AccentColor);
            ColorScheme.Initialize(accentColor, pilotServices.PilotDialogService.Theme);
        }

        public void Build(IMenuBuilder builder, TasksViewContext2 context)
        {
            builder.AddSeparator(2);

            var menu = builder.AddItem(CREATE_NEW_TASK, 3)
                .WithHeader("Create new task")
                .WithIcon(IconsService.CreateTaskIcon)
                .WithSubmenu();

            var taskTypes = _taskTypesService.RootTaskTypes;
            BuildSubMenu(taskTypes, menu);

            var wfMenu = builder.AddItem(CREATE_NEW_WORKFLOW, 4)
                .WithHeader("Create new workflow")
                .WithIcon(IconsService.CreateWorkflowIcon)
                .WithSubmenu();

            var workflowTypes = _taskTypesService.RootWorkflowTypes;
            BuildSubMenu(workflowTypes, wfMenu);

            builder.AddItem(SHOW_TASK_DIALOG_WITH_PREDEFINED_DATA, 5)
                .WithHeader("Show task dialog with predefined data")
                .WithIcon(IconsService.CreateTaskIcon);

            builder.AddItem(SHOW_WORKFLOW_DIALOG_WITH_PREDEFINED_DATA, 6)
                .WithHeader("Show workflow dialog with predefined data")
                .WithIcon(IconsService.CreateWorkflowIcon);

            if (builder.ItemNames.Count() > 6)
                builder.AddSeparator(7);
        }

        public void OnMenuItemClick(string name, TasksViewContext2 context)
        {
            if (name == SHOW_WORKFLOW_DIALOG_WITH_PREDEFINED_DATA)
            {
                _dialogService.ShowWorkflowDialogWithData(_taskTypesService);
                return;
            }

            if (name == SHOW_TASK_DIALOG_WITH_PREDEFINED_DATA)
            {
                _dialogService.ShowTaskDialogWithData(_taskTypesService);
                return;
            }

            if (!_types.TryGetValue(name, out var type))
                return;

            if (type.IsWorkflowType())
            {
                _dialogService.ShowNewWorkflowDialog(type, _taskTypesService);
                return;
            }

            if (type.IsTaskType())
            {
                _dialogService.ShowNewTaskDialog(type);
            }
        }

        private void BuildSubMenu(IReadOnlyList<IType> types, IMenuBuilder menu)
        {
            for (var i = 0; i < types.Count; i++)
            {
                var type = types[i];
                var typeName = type.Name.GetXamlName();
                menu.AddItem(typeName, i).WithHeader(type.Title).WithIcon(type.SvgIcon);
                _types[typeName] = type;
            }
        }
    }
}