﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reactive.Disposables;
using System.Reactive.Linq;
using System.Windows.Threading;
using Ascon.Pilot.SDK;
using DynamicData;
using ProjectCloneWizard.Data;

namespace ProjectCloneWizard.ObjectsTree
{
    public class DataObjectService : IDataObjectService
    {
        private readonly IObjectsRepository _repository;
        private readonly IChildrenFilter _childrenFilter;
        private readonly CompositeDisposable _cleanUp = new CompositeDisposable();
        private readonly SourceCache<IDataObject, Guid> _dataObjects = new SourceCache<IDataObject, Guid>(x => x.Id);

        public IObservableCache<IDataObject, Guid> DataObjects => _dataObjects.AsObservableCache();

        public DataObjectService(IObjectsRepository repository, IChildrenFilter childrenFilter)
        {
            _repository = repository;
            _childrenFilter = childrenFilter;
            _cleanUp.Add(_dataObjects);
        }

        public void Load(Guid id, Action nothingLoaded, bool isAddParent)
        {
            var disposable = _repository.SubscribeObjects(new []{ id })
                .Where(x => _childrenFilter.Filter(x))
                .SubscribeOnDispatcher(DispatcherPriority.Background)
                .Subscribe(o =>
                {
                    var children = _childrenFilter.ChildrenFilter(o).ToList();
                    if (children.Count == 0)
                    {
                        nothingLoaded();
                        return;
                    }
                    if (isAddParent)
                    {
                        _dataObjects.AddOrUpdate(o);
                    }

                    Load(children);
                });

            _cleanUp.Add(disposable);
        }

        public void Dispose()
        {
            _cleanUp.Dispose();
        }

        private void Load(IEnumerable<Guid> ids)
        {
            var disposable = _repository.SubscribeObjects(ids)
                .Where(x => _childrenFilter.Filter(x))
                .SubscribeOnDispatcher(DispatcherPriority.Background)
                .Subscribe(o =>
                {
                    _dataObjects.AddOrUpdate(o);
                });
            _cleanUp.Add(disposable);
        }
    }
}
