﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Ascon.Pilot.SDK;
using Ascon.Pilot.SDK.Toolbar;
using ProjectCloneWizard.Common;
using ProjectCloneWizard.Data;

namespace ProjectCloneWizard
{
    [Export(typeof(IToolbar<ObjectsViewContext>))]
    public class ProjectCloneToolbar : ToolbarItemSubmenuHandler, IToolbar<ObjectsViewContext>
    {
        private readonly ProjectCloneCreator _projectCloneCreator;
        private IDataObject _selected;

        [ImportingConstructor]
        public ProjectCloneToolbar(IObjectsRepository repository, IPilotDialogService pilotDialogService, IAttributeFormatParser attributeFormatParser, 
            IObjectModifier modifier, IFileProvider fileProvider, ITabServiceProvider tabServiceProvider)
        {
            _projectCloneCreator = new ProjectCloneCreator(repository, pilotDialogService, attributeFormatParser, modifier, fileProvider, tabServiceProvider);
        }

        public void Build(IToolbarBuilder builder, ObjectsViewContext context)
        {
            var objects = context.SelectedObjects.ToList();
            if (objects.Count != 1)
                return;

            _selected = objects.First();

            var menu = builder.ItemNames.ToList();
            if (!menu.Contains("miCreate"))
                return;

            builder.ReplaceMenuButtonItem("miCreate").WithMenu(this);
        }

        public void OnToolbarItemClick(string name, ObjectsViewContext context)
        {
            var selected = context.SelectedObjects.First();
            _projectCloneCreator.ShowWizardDialog(selected);
        }

        public override void OnSubmenuRequested(IToolbarBuilder builder)
        {
            var itemNames = builder.ItemNames.ToList();
            var index = itemNames.Count;

            var smartFolderIndex = itemNames.IndexOf("miCreateSmartfoldertype");
            if (smartFolderIndex != -1)
                index = smartFolderIndex - 1;

            if (_selected.Type.IsProject)
            {
                builder.AddButtonItem(Const.COMMAND_UPDATE_NAME, index)
                       .WithHeader(Const.COMMAND_UPDATE_TITLE)
                       .WithIcon(IconLoader.GetIcon(_projectCloneCreator.Icon));
            }
            else
            {
                builder.AddButtonItem(Const.COMMAND_NAME, index)
                       .WithHeader(Const.COMMAND_TITLE)
                       .WithIcon(IconLoader.GetIcon(_projectCloneCreator.Icon));
            }
        }
    }
}
